function ReadMe()
% "ReadMe" explains on when to use a matrix, and when and how a function.
% 
% Krylov subspace methods for solving A*x=b rely on vectors produced by
% repeatedly multiplying vectors by A. Here, A is an n x n matrix and b is
% a given right-hand side vector of dimension n, x is the n-vector that has
% to be computed (to some accuracy).
% 
% Actually, Krylov subspace methods do not need the matrix A explicitly.
% They only need the results c of the multiplication of vectors v by A:
% c=A*u. These results can also be produced by functions as c=MV(u). Often
% this is more efficient. For instance, if A is defined as A=L\(H/U),
% where L and U are tri-diagonal, L lower triangular and U upper
% triangular, and H is 5-diagonal, then, generally, A will be a ful
% matrix. Computing c=A*u, using A, will require 2*n*n flop, whereas
% computing c as c=MV(u) requires 23*n flop if
% 
%     function c=MV(u)
%        c=L\(H*(U\u));
%     end
%     
% All of our iterative methods here are coded such that they except a
% matrix, say, A as input but also a function, say, MV. In case of a
% function, the input is either the handle of the function (that is, in
% C++ terminology, the pointer to a function) or the name of the function.
% So, in a call like
% 
% >  x=gcr(A,b)
% 
% A is either  a) an n x n matrix, or
%              b) a function handle (as A=@MV), or
%              c) the name of a function (as A='MV')
%              
% Using a function needs extra care to avoid repeatedly defining the
% ingredients. For instance, if, in the above example, say,
% H=gallery('poisson',200), L=tril(H), and U=triu(U) then the function
% 
%     function c=MV(u)
%        H=gallery('poisson',200); L=tril(H); U=triu(U);
%        c=L\(H*(U\u));
%     end
% 
% would be correctly defined, but extremely inefficient, since H, L and U
% would be formed at each call of the function MV. Passing the ingredients
% to the solver, as x=gcr(H,L,U,b), would require to modify the function
% gcr, which is not very practical (if several different problems are to
% be solved, possibly using different solvers). 
% 
% There are two ways to pass the ingredients, as H, L, and U, to the
% function MV.
% b) Use nested functions, where the function MV is defined inside another
% function in which H, U and L have been defined and the handle @MV is the
% output of the outer function (see MATLAB's documentation "Using Handles
% to Store Function Parameters").
% c) Use global variables: define H,L,U and declare them to be global also
% in the function MV. (The use of global variables is not recommended. If
% used, avoid mixing up variables, by giving complicated names to global
% variables, as MyMATRIX_H rather than H).
% 
% The files problem1a.m, problem1b.m, and problem1c.m define the same 
% problem A*x=b (the same matrix), but in three different ways, in the
% ways described above.
%
%
% See also   problem0, problem0a, problem1a, problem1b, problem1c

             
























